import os

class DownloadParameters:
    """ Describes the related parameters when downloading file from server.

    such as the type of entities and data scope that you want to download.
    """

    def __init__(self,
                 result_file_directory=None,
                 result_file_name=None,
                 overwrite_result_file=False,
                 data_scope=None,
                 download_entities=None,
                 file_type='Csv',
                 campaign_ids=None,
                 last_sync_time_in_utc=None,
                 timeout_in_milliseconds=None):
        """ Initialize an instance of this class.

        :param result_file_directory: (optional) The directory where the file will be downloaded.
        :type result_file_directory: str
        :param result_file_name: (optional) The name of the download result file.
        :type result_file_name: str
        :param overwrite_result_file: (optional) Whether the local result file should be overwritten if it already exists, default is False.
        :type overwrite_result_file: bool
        :param data_scope: (optional) The scope or types of data to download.
                            For possible values, see DataScope Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type data_scope: list[str]
        :param download_entities: (optional) The type of entities to download.
                            For possible values, see BulkDownloadEntity Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type download_entities: list[str]
        :param file_type: (optional) The extension type of the downloaded file.
                            For possible values, see DownloadFileType Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type file_type: str
        :param campaign_ids: (optional) The campaigns to download.
                            You can specify a maximum of 1,000 campaigns. The campaigns that you specify must belong to the same account.
        :type campaign_ids: list[int]
        :param last_sync_time_in_utc: (optional) The last time that you requested a download.
                            The date and time is expressed in Coordinated Universal Time (UTC).
                            Typically, you request a full download the first time you call the operation by setting this element to null.
                            On all subsequent calls you set the last sync time to the time stamp of the previous download.
                            The download file contains the time stamp of the download in the SyncTime column of the Account record.
                            Use the time stamp to set LastSyncTimeInUTC the next time that you request a download.
                            If you specify the last sync time, only those entities that have changed (been updated or deleted)
                            since the specified date and time will be downloaded. However, if the campaign data has not been previously downloaded,
                            the operation performs a full download.
        :type last_sync_time_in_utc: datetime
        :param timeout_in_milliseconds: (optional) timeout for bulk download operations in milliseconds
        :type timeout_in_milliseconds: int
        """

        self._result_file_directory = result_file_directory
        self._result_file_name = result_file_name
        self._decompress_result_file = True
        if result_file_name is not None:
            _, ext = os.path.splitext(result_file_name)
            if ext == '.zip':
                self._decompress_result_file = False
        self._overwrite_result_file = overwrite_result_file
        self._submit_download_parameter = SubmitDownloadParameters(
            data_scope=data_scope,
            download_entities=download_entities,
            file_type=file_type,
            campaign_ids=campaign_ids,
            last_sync_time_in_utc=last_sync_time_in_utc,
        )
        self._timeout_in_milliseconds = timeout_in_milliseconds


    @property
    def result_file_directory(self):
        """ The directory where the file will be downloaded.

        :rtype: str
        """

        return self._result_file_directory

    @property
    def result_file_name(self):
        """ The name of the download result file.

        :rtype: str
        """

        return self._result_file_name

    @property
    def overwrite_result_file(self):
        """ Whether the local result file should be overwritten if it already exists.

        :rtype: bool
        """

        return self._overwrite_result_file

    @result_file_directory.setter
    def result_file_directory(self, result_file_directory):
        self._result_file_directory = result_file_directory

    @result_file_name.setter
    def result_file_name(self, result_file_name):
        self._result_file_name = result_file_name

    @overwrite_result_file.setter
    def overwrite_result_file(self, overwrite):
        self._overwrite_result_file = overwrite

    @property
    def decompress_result_file(self):
        """ If need to decompress the result file after download.

        This property is determined by the result_file_name, by default will do decompression.
        if the result_file_name has the extension of '.zip' then do not do decompression.

        :rtype: bool
        """

        return self._decompress_result_file

    @property
    def data_scope(self):
        """ The scope or types of data to download.

        For possible values, see DataScope Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: list[str]
        """

        return self._submit_download_parameter.data_scope

    @data_scope.setter
    def data_scope(self, value):
        self._submit_download_parameter.data_scope = value

    @property
    def download_entities(self):
        """ The type of entities to download.

        For possible values, see BulkDownloadEntity Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: list[str]
        """

        return self._submit_download_parameter.download_entities

    @download_entities.setter
    def download_entities(self, value):
        self._submit_download_parameter.download_entities = value

    @property
    def file_type(self):
        """ The extension type of the downloaded file.

        For possible values, see DownloadFileType Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: str
        """

        return self._submit_download_parameter.file_type

    @file_type.setter
    def file_type(self, value):
        self._submit_download_parameter.file_type = value

    @property
    def campaign_ids(self):
        """ The campaigns to download.

        You can specify a maximum of 1,000 campaigns. The campaigns that you specify must belong to the same account.

        :rtype: list[int]
        """

        return self._submit_download_parameter.campaign_ids

    @campaign_ids.setter
    def campaign_ids(self, value):
        self._submit_download_parameter.campaign_ids = value

    @property
    def last_sync_time_in_utc(self):
        """ The last time that you requested a download.

        The date and time is expressed in Coordinated Universal Time (UTC).
        Typically, you request a full download the first time you call the operation by setting this element to null.
        On all subsequent calls you set the last sync time to the time stamp of the previous download.
        The download file contains the time stamp of the download in the SyncTime column of the Account record.
        Use the time stamp to set LastSyncTimeInUTC the next time that you request a download.
        If you specify the last sync time, only those entities that have changed (been updated or deleted)
        since the specified date and time will be downloaded. However, if the campaign data has not been previously downloaded,
        the operation performs a full download.

        :rtype: datetime
        """

        return self._submit_download_parameter.last_sync_time_in_utc

    @last_sync_time_in_utc.setter
    def last_sync_time_in_utc(self, value):
        self._submit_download_parameter.last_sync_time_in_utc = value

    @property
    def timeout_in_milliseconds(self):
        return self._timeout_in_milliseconds

    @timeout_in_milliseconds.setter
    def timeout_in_milliseconds(self, value):
        self._timeout_in_milliseconds = value


class SubmitDownloadParameters(object):
    """ Describes the service request parameters such as the type of entities and data scope that you want to download. """

    def __init__(self,
                 data_scope=None,
                 download_entities=None,
                 file_type='Csv',
                 campaign_ids=None,
                 last_sync_time_in_utc=None):
        """ Initialize an object of this class.

        :param data_scope: (optional) The scope or types of data to download.
                            For possible values, see DataScope Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type data_scope: list[str]
        :param download_entities: (optional) The type of entities to download.
                            For possible values, see BulkDownloadEntity Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type download_entities: list[str]
        :param file_type: (optional) The extension type of the downloaded file.
                            For possible values, see DownloadFileType Value Set at https://go.microsoft.com/fwlink/?linkid=846127.
        :type file_type: str
        :param campaign_ids: (optional) The campaigns to download.
                            You can specify a maximum of 1,000 campaigns. The campaigns that you specify must belong to the same account.
        :type campaign_ids: list[int]
        :param last_sync_time_in_utc: (optional) The last time that you requested a download.
                            The date and time is expressed in Coordinated Universal Time (UTC).
                            Typically, you request a full download the first time you call the operation by setting this element to null.
                            On all subsequent calls you set the last sync time to the time stamp of the previous download.
                            The download file contains the time stamp of the download in the SyncTime column of the Account record.
                            Use the time stamp to set LastSyncTimeInUTC the next time that you request a download.
                            If you specify the last sync time, only those entities that have changed (been updated or deleted)
                            since the specified date and time will be downloaded. However, if the campaign data has not been previously downloaded,
                            the operation performs a full download.
        :type last_sync_time_in_utc: datetime
        """

        self._data_scope = data_scope
        self._download_entities = download_entities
        self._file_type = file_type
        self._campaign_ids = campaign_ids
        self._last_sync_time_in_utc = last_sync_time_in_utc

    @property
    def data_scope(self):
        """ The scope or types of data to download.

        For possible values, see DataScope Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: list[str]
        """

        return self._data_scope

    @data_scope.setter
    def data_scope(self, value):
        self._data_scope = value

    @property
    def download_entities(self):
        """ The type of entities to download.

        For possible values, see BulkDownloadEntity Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: list[str]
        """

        return self._download_entities

    @download_entities.setter
    def download_entities(self, value):
        self._download_entities = value

    @property
    def file_type(self):
        """ The extension type of the downloaded file.

        For possible values, see DownloadFileType Value Set at https://go.microsoft.com/fwlink/?linkid=846127.

        :rtype: str
        """

        return self._file_type

    @file_type.setter
    def file_type(self, value):
        self._file_type = value

    @property
    def campaign_ids(self):
        """ The campaigns to download.

        You can specify a maximum of 1,000 campaigns. The campaigns that you specify must belong to the same account.

        :rtype: list[int]
        """

        return self._campaign_ids

    @campaign_ids.setter
    def campaign_ids(self, value):
        self._campaign_ids = value

    @property
    def last_sync_time_in_utc(self):
        """ The last time that you requested a download.

        The date and time is expressed in Coordinated Universal Time (UTC).
        Typically, you request a full download the first time you call the operation by setting this element to null.
        On all subsequent calls you set the last sync time to the time stamp of the previous download.
        The download file contains the time stamp of the download in the SyncTime column of the Account record.
        Use the time stamp to set LastSyncTimeInUTC the next time that you request a download.
        If you specify the last sync time, only those entities that have changed (been updated or deleted)
        since the specified date and time will be downloaded. However, if the campaign data has not been previously downloaded,
        the operation performs a full download.

        :rtype: datetime
        """

        return self._last_sync_time_in_utc

    @last_sync_time_in_utc.setter
    def last_sync_time_in_utc(self, value):
        self._last_sync_time_in_utc = value



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *
from .bulk_audience import BulkAudience

class BulkSimilarRemarketingList(BulkAudience):
    """ Represents an Similar Remarketing List that can be read or written in a bulk file.

    This class exposes the :attr:`similar_remarketing_list` property that can be read and written as fields of the
    Similar Remarketing List record in a bulk file.

    For more information, see Similar Remarketing List at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 similar_remarketing_list=None,
                 status=None,):
        super(BulkSimilarRemarketingList, self).__init__(audience = similar_remarketing_list, status = status)

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.SourceId,
            field_to_csv=lambda c: bulk_str(c.similar_remarketing_list.SourceId),
            csv_to_field=lambda c, v: setattr(c.similar_remarketing_list, 'SourceId', int(v) if v else None)
        ),
    ]

    @property
    def similar_remarketing_list(self):
        """ Defines a Similar Remarketing List """

        return self._audience

    @similar_remarketing_list.setter
    def similar_remarketing_list(self, similar_remarketing_list):
        self._audience = similar_remarketing_list

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.similar_remarketing_list, 'similar_remarketing_list')
        super(BulkSimilarRemarketingList, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkSimilarRemarketingList._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self.similar_remarketing_list = _CAMPAIGN_OBJECT_FACTORY_V13.create('SimilarRemarketingList')
        super(BulkSimilarRemarketingList, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkSimilarRemarketingList._MAPPINGS)




# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.client_link_status import ClientLinkStatus
from openapi_client.models.customer.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from typing_extensions import Self

class ClientLink(BaseModel):
    """
    ClientLink
    """ # noqa: E501
    type: Optional[StrictStr] = Field(default=None, alias="Type")
    client_entity_id: Optional[StrictStr] = Field(default=None, alias="ClientEntityId")
    client_entity_number: Optional[StrictStr] = Field(default=None, alias="ClientEntityNumber")
    client_entity_name: Optional[StrictStr] = Field(default=None, alias="ClientEntityName")
    managing_customer_id: Optional[StrictStr] = Field(default=None, alias="ManagingCustomerId")
    managing_customer_number: Optional[StrictStr] = Field(default=None, alias="ManagingCustomerNumber")
    managing_customer_name: Optional[StrictStr] = Field(default=None, alias="ManagingCustomerName")
    note: Optional[StrictStr] = Field(default=None, alias="Note")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    inviter_email: Optional[StrictStr] = Field(default=None, alias="InviterEmail")
    inviter_name: Optional[StrictStr] = Field(default=None, alias="InviterName")
    inviter_phone: Optional[StrictStr] = Field(default=None, alias="InviterPhone")
    is_bill_to_client: Optional[StrictBool] = Field(default=None, alias="IsBillToClient")
    start_date: Optional[datetime] = Field(default=None, alias="StartDate")
    status: Optional[ClientLinkStatus] = Field(default=None, alias="Status")
    suppress_notification: Optional[StrictBool] = Field(default=None, alias="SuppressNotification")
    last_modified_date_time: Optional[datetime] = Field(default=None, alias="LastModifiedDateTime")
    last_modified_by_user_id: Optional[StrictStr] = Field(default=None, alias="LastModifiedByUserId")
    timestamp: Optional[StrictStr] = Field(default=None, alias="Timestamp")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    customer_link_permission: Optional[StrictStr] = Field(default=None, alias="CustomerLinkPermission")
    __properties: ClassVar[List[str]] = ["Type", "ClientEntityId", "ClientEntityNumber", "ClientEntityName", "ManagingCustomerId", "ManagingCustomerNumber", "ManagingCustomerName", "Note", "Name", "InviterEmail", "InviterName", "InviterPhone", "IsBillToClient", "StartDate", "Status", "SuppressNotification", "LastModifiedDateTime", "LastModifiedByUserId", "Timestamp", "ForwardCompatibilityMap", "CustomerLinkPermission"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        # set to None if client_entity_id (nullable) is None
        # and model_fields_set contains the field
        if self.client_entity_id is None and "client_entity_id" in self.model_fields_set:
            _dict['ClientEntityId'] = None

        # set to None if client_entity_number (nullable) is None
        # and model_fields_set contains the field
        if self.client_entity_number is None and "client_entity_number" in self.model_fields_set:
            _dict['ClientEntityNumber'] = None

        # set to None if client_entity_name (nullable) is None
        # and model_fields_set contains the field
        if self.client_entity_name is None and "client_entity_name" in self.model_fields_set:
            _dict['ClientEntityName'] = None

        # set to None if managing_customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.managing_customer_id is None and "managing_customer_id" in self.model_fields_set:
            _dict['ManagingCustomerId'] = None

        # set to None if managing_customer_number (nullable) is None
        # and model_fields_set contains the field
        if self.managing_customer_number is None and "managing_customer_number" in self.model_fields_set:
            _dict['ManagingCustomerNumber'] = None

        # set to None if managing_customer_name (nullable) is None
        # and model_fields_set contains the field
        if self.managing_customer_name is None and "managing_customer_name" in self.model_fields_set:
            _dict['ManagingCustomerName'] = None

        # set to None if note (nullable) is None
        # and model_fields_set contains the field
        if self.note is None and "note" in self.model_fields_set:
            _dict['Note'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if inviter_email (nullable) is None
        # and model_fields_set contains the field
        if self.inviter_email is None and "inviter_email" in self.model_fields_set:
            _dict['InviterEmail'] = None

        # set to None if inviter_name (nullable) is None
        # and model_fields_set contains the field
        if self.inviter_name is None and "inviter_name" in self.model_fields_set:
            _dict['InviterName'] = None

        # set to None if inviter_phone (nullable) is None
        # and model_fields_set contains the field
        if self.inviter_phone is None and "inviter_phone" in self.model_fields_set:
            _dict['InviterPhone'] = None

        # set to None if is_bill_to_client (nullable) is None
        # and model_fields_set contains the field
        if self.is_bill_to_client is None and "is_bill_to_client" in self.model_fields_set:
            _dict['IsBillToClient'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if suppress_notification (nullable) is None
        # and model_fields_set contains the field
        if self.suppress_notification is None and "suppress_notification" in self.model_fields_set:
            _dict['SuppressNotification'] = None

        # set to None if last_modified_date_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_date_time is None and "last_modified_date_time" in self.model_fields_set:
            _dict['LastModifiedDateTime'] = None

        # set to None if last_modified_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_by_user_id is None and "last_modified_by_user_id" in self.model_fields_set:
            _dict['LastModifiedByUserId'] = None

        # set to None if timestamp (nullable) is None
        # and model_fields_set contains the field
        if self.timestamp is None and "timestamp" in self.model_fields_set:
            _dict['Timestamp'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        # set to None if customer_link_permission (nullable) is None
        # and model_fields_set contains the field
        if self.customer_link_permission is None and "customer_link_permission" in self.model_fields_set:
            _dict['CustomerLinkPermission'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ClientLink from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Type": obj.get("Type") if obj.get("Type") is not None else None,
                        "ClientEntityId": obj.get("ClientEntityId") if obj.get("ClientEntityId") is not None else None,
                        "ClientEntityNumber": obj.get("ClientEntityNumber") if obj.get("ClientEntityNumber") is not None else None,
                        "ClientEntityName": obj.get("ClientEntityName") if obj.get("ClientEntityName") is not None else None,
                        "ManagingCustomerId": obj.get("ManagingCustomerId") if obj.get("ManagingCustomerId") is not None else None,
                        "ManagingCustomerNumber": obj.get("ManagingCustomerNumber") if obj.get("ManagingCustomerNumber") is not None else None,
                        "ManagingCustomerName": obj.get("ManagingCustomerName") if obj.get("ManagingCustomerName") is not None else None,
                        "Note": obj.get("Note") if obj.get("Note") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "InviterEmail": obj.get("InviterEmail") if obj.get("InviterEmail") is not None else None,
                        "InviterName": obj.get("InviterName") if obj.get("InviterName") is not None else None,
                        "InviterPhone": obj.get("InviterPhone") if obj.get("InviterPhone") is not None else None,
                        "IsBillToClient": obj.get("IsBillToClient") if obj.get("IsBillToClient") is not None else None,
                        "StartDate": obj.get("StartDate") if obj.get("StartDate") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "SuppressNotification": obj.get("SuppressNotification") if obj.get("SuppressNotification") is not None else None,
                        "LastModifiedDateTime": obj.get("LastModifiedDateTime") if obj.get("LastModifiedDateTime") is not None else None,
                        "LastModifiedByUserId": obj.get("LastModifiedByUserId") if obj.get("LastModifiedByUserId") is not None else None,
                        "Timestamp": obj.get("Timestamp") if obj.get("Timestamp") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None,
                        "CustomerLinkPermission": obj.get("CustomerLinkPermission") if obj.get("CustomerLinkPermission") is not None else None
        })
        return _obj



# coding: utf-8

# flake8: noqa
"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.customer.account_additional_field import AccountAdditionalField
from openapi_client.models.customer.account_financial_status import AccountFinancialStatus
from openapi_client.models.customer.account_info import AccountInfo
from openapi_client.models.customer.account_info_with_customer_data import AccountInfoWithCustomerData
from openapi_client.models.customer.account_life_cycle_status import AccountLifeCycleStatus
from openapi_client.models.customer.account_tax_certificate import AccountTaxCertificate
from openapi_client.models.customer.ad_api_error import AdApiError
from openapi_client.models.customer.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.customer.add_account_request import AddAccountRequest
from openapi_client.models.customer.add_account_response import AddAccountResponse
from openapi_client.models.customer.add_client_links_request import AddClientLinksRequest
from openapi_client.models.customer.add_client_links_response import AddClientLinksResponse
from openapi_client.models.customer.add_prepay_account_request import AddPrepayAccountRequest
from openapi_client.models.customer.add_prepay_account_response import AddPrepayAccountResponse
from openapi_client.models.customer.address import Address
from openapi_client.models.customer.advertiser_account import AdvertiserAccount
from openapi_client.models.customer.api_batch_fault import ApiBatchFault
from openapi_client.models.customer.api_fault import ApiFault
from openapi_client.models.customer.application_fault import ApplicationFault
from openapi_client.models.customer.auto_tag_type import AutoTagType
from openapi_client.models.customer.batch_error import BatchError
from openapi_client.models.customer.client_link import ClientLink
from openapi_client.models.customer.client_link_status import ClientLinkStatus
from openapi_client.models.customer.contact_info import ContactInfo
from openapi_client.models.customer.currency_code import CurrencyCode
from openapi_client.models.customer.customer import Customer
from openapi_client.models.customer.customer_financial_status import CustomerFinancialStatus
from openapi_client.models.customer.customer_info import CustomerInfo
from openapi_client.models.customer.customer_life_cycle_status import CustomerLifeCycleStatus
from openapi_client.models.customer.customer_role import CustomerRole
from openapi_client.models.customer.date_range import DateRange
from openapi_client.models.customer.delete_account_request import DeleteAccountRequest
from openapi_client.models.customer.delete_customer_request import DeleteCustomerRequest
from openapi_client.models.customer.delete_user_request import DeleteUserRequest
from openapi_client.models.customer.dismiss_notifications_request import DismissNotificationsRequest
from openapi_client.models.customer.email_format import EmailFormat
from openapi_client.models.customer.find_accounts_or_customers_info_request import FindAccountsOrCustomersInfoRequest
from openapi_client.models.customer.find_accounts_or_customers_info_response import FindAccountsOrCustomersInfoResponse
from openapi_client.models.customer.find_accounts_request import FindAccountsRequest
from openapi_client.models.customer.find_accounts_response import FindAccountsResponse
from openapi_client.models.customer.get_accessible_customer_request import GetAccessibleCustomerRequest
from openapi_client.models.customer.get_accessible_customer_response import GetAccessibleCustomerResponse
from openapi_client.models.customer.get_account_pilot_features_request import GetAccountPilotFeaturesRequest
from openapi_client.models.customer.get_account_pilot_features_response import GetAccountPilotFeaturesResponse
from openapi_client.models.customer.get_account_request import GetAccountRequest
from openapi_client.models.customer.get_account_response import GetAccountResponse
from openapi_client.models.customer.get_accounts_info_request import GetAccountsInfoRequest
from openapi_client.models.customer.get_accounts_info_response import GetAccountsInfoResponse
from openapi_client.models.customer.get_current_user_response import GetCurrentUserResponse
from openapi_client.models.customer.get_customer_pilot_features_request import GetCustomerPilotFeaturesRequest
from openapi_client.models.customer.get_customer_pilot_features_response import GetCustomerPilotFeaturesResponse
from openapi_client.models.customer.get_customer_request import GetCustomerRequest
from openapi_client.models.customer.get_customer_response import GetCustomerResponse
from openapi_client.models.customer.get_customers_info_request import GetCustomersInfoRequest
from openapi_client.models.customer.get_customers_info_response import GetCustomersInfoResponse
from openapi_client.models.customer.get_linked_accounts_and_customers_info_request import GetLinkedAccountsAndCustomersInfoRequest
from openapi_client.models.customer.get_linked_accounts_and_customers_info_response import GetLinkedAccountsAndCustomersInfoResponse
from openapi_client.models.customer.get_notifications_request import GetNotificationsRequest
from openapi_client.models.customer.get_notifications_response import GetNotificationsResponse
from openapi_client.models.customer.get_pilot_features_countries_response import GetPilotFeaturesCountriesResponse
from openapi_client.models.customer.get_user_mfa_status_response import GetUserMFAStatusResponse
from openapi_client.models.customer.get_user_request import GetUserRequest
from openapi_client.models.customer.get_user_response import GetUserResponse
from openapi_client.models.customer.get_users_info_request import GetUsersInfoRequest
from openapi_client.models.customer.get_users_info_response import GetUsersInfoResponse
from openapi_client.models.customer.industry import Industry
from openapi_client.models.customer.key_value_entity_oflong_andint import KeyValueEntityOflongAndint
from openapi_client.models.customer.key_value_entity_ofstring_andstring import KeyValueEntityOfstringAndstring
from openapi_client.models.customer.key_value_pair_ofstring_and_byte_array import KeyValuePairOfstringAndByteArray
from openapi_client.models.customer.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.customer.lcid import LCID
from openapi_client.models.customer.language_type import LanguageType
from openapi_client.models.customer.map_account_id_to_external_account_ids_request import MapAccountIdToExternalAccountIdsRequest
from openapi_client.models.customer.map_customer_id_to_external_customer_id_request import MapCustomerIdToExternalCustomerIdRequest
from openapi_client.models.customer.notification import Notification
from openapi_client.models.customer.operation_error import OperationError
from openapi_client.models.customer.order_by import OrderBy
from openapi_client.models.customer.order_by_field import OrderByField
from openapi_client.models.customer.paging import Paging
from openapi_client.models.customer.payment_method_type import PaymentMethodType
from openapi_client.models.customer.person_name import PersonName
from openapi_client.models.customer.pilot_feature import PilotFeature
from openapi_client.models.customer.predicate import Predicate
from openapi_client.models.customer.predicate_operator import PredicateOperator
from openapi_client.models.customer.search_accounts_request import SearchAccountsRequest
from openapi_client.models.customer.search_accounts_response import SearchAccountsResponse
from openapi_client.models.customer.search_client_links_request import SearchClientLinksRequest
from openapi_client.models.customer.search_client_links_response import SearchClientLinksResponse
from openapi_client.models.customer.search_customers_request import SearchCustomersRequest
from openapi_client.models.customer.search_customers_response import SearchCustomersResponse
from openapi_client.models.customer.search_user_invitations_request import SearchUserInvitationsRequest
from openapi_client.models.customer.search_user_invitations_response import SearchUserInvitationsResponse
from openapi_client.models.customer.secret_question import SecretQuestion
from openapi_client.models.customer.send_user_invitation_request import SendUserInvitationRequest
from openapi_client.models.customer.send_user_invitation_response import SendUserInvitationResponse
from openapi_client.models.customer.service_level import ServiceLevel
from openapi_client.models.customer.signup_customer_request import SignupCustomerRequest
from openapi_client.models.customer.signup_customer_response import SignupCustomerResponse
from openapi_client.models.customer.sort_order import SortOrder
from openapi_client.models.customer.tax_certificate_status import TaxCertificateStatus
from openapi_client.models.customer.time_zone_type import TimeZoneType
from openapi_client.models.customer.update_account_request import UpdateAccountRequest
from openapi_client.models.customer.update_account_response import UpdateAccountResponse
from openapi_client.models.customer.update_client_links_request import UpdateClientLinksRequest
from openapi_client.models.customer.update_client_links_response import UpdateClientLinksResponse
from openapi_client.models.customer.update_customer_request import UpdateCustomerRequest
from openapi_client.models.customer.update_customer_response import UpdateCustomerResponse
from openapi_client.models.customer.update_prepay_account_request import UpdatePrepayAccountRequest
from openapi_client.models.customer.update_prepay_account_response import UpdatePrepayAccountResponse
from openapi_client.models.customer.update_user_request import UpdateUserRequest
from openapi_client.models.customer.update_user_response import UpdateUserResponse
from openapi_client.models.customer.update_user_roles_request import UpdateUserRolesRequest
from openapi_client.models.customer.update_user_roles_response import UpdateUserRolesResponse
from openapi_client.models.customer.upgrade_customer_to_agency_request import UpgradeCustomerToAgencyRequest
from openapi_client.models.customer.user import User
from openapi_client.models.customer.user_info import UserInfo
from openapi_client.models.customer.user_invitation import UserInvitation
from openapi_client.models.customer.user_life_cycle_status import UserLifeCycleStatus
from openapi_client.models.customer.validate_address_request import ValidateAddressRequest
from openapi_client.models.customer.validate_address_response import ValidateAddressResponse



from abc import ABCMeta, abstractproperty, abstractmethod

from bingads.v13.bulk.entities import BulkError
from bingads.v13.internal.bulk.bulk_object import _BulkObject


class _BulkEntityIdentifier(_BulkObject, metaclass=ABCMeta):

    @abstractproperty
    def is_delete_row(self):
        raise NotImplementedError()

    @abstractmethod
    def _create_entity_with_this_identifier(self):
        raise NotImplementedError()

    def write_to_stream(self, row_writer, exclude_readonly_data):
        row_writer.write_object_row(self)

    def read_related_data_from_stream(self, stream_reader):
        if self.is_delete_row:
            has_more_errors = True

            while has_more_errors:
                has_more_errors, error = stream_reader.try_read(BulkError)

    @property
    def can_enclose_in_multiline_entity(self):
        return True

    def enclose_in_multiline_entity(self):
        return self._create_entity_with_this_identifier()



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class Address(BaseModel):
    """
    Address
    """ # noqa: E501
    street_address: Optional[StrictStr] = Field(default=None, alias="StreetAddress")
    street_address2: Optional[StrictStr] = Field(default=None, alias="StreetAddress2")
    city_name: Optional[StrictStr] = Field(default=None, alias="CityName")
    province_code: Optional[StrictStr] = Field(default=None, alias="ProvinceCode")
    province_name: Optional[StrictStr] = Field(default=None, alias="ProvinceName")
    postal_code: Optional[StrictStr] = Field(default=None, alias="PostalCode")
    country_code: Optional[StrictStr] = Field(default=None, alias="CountryCode")
    __properties: ClassVar[List[str]] = ["StreetAddress", "StreetAddress2", "CityName", "ProvinceCode", "ProvinceName", "PostalCode", "CountryCode"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if street_address (nullable) is None
        # and model_fields_set contains the field
        if self.street_address is None and "street_address" in self.model_fields_set:
            _dict['StreetAddress'] = None

        # set to None if street_address2 (nullable) is None
        # and model_fields_set contains the field
        if self.street_address2 is None and "street_address2" in self.model_fields_set:
            _dict['StreetAddress2'] = None

        # set to None if city_name (nullable) is None
        # and model_fields_set contains the field
        if self.city_name is None and "city_name" in self.model_fields_set:
            _dict['CityName'] = None

        # set to None if province_code (nullable) is None
        # and model_fields_set contains the field
        if self.province_code is None and "province_code" in self.model_fields_set:
            _dict['ProvinceCode'] = None

        # set to None if province_name (nullable) is None
        # and model_fields_set contains the field
        if self.province_name is None and "province_name" in self.model_fields_set:
            _dict['ProvinceName'] = None

        # set to None if postal_code (nullable) is None
        # and model_fields_set contains the field
        if self.postal_code is None and "postal_code" in self.model_fields_set:
            _dict['PostalCode'] = None

        # set to None if country_code (nullable) is None
        # and model_fields_set contains the field
        if self.country_code is None and "country_code" in self.model_fields_set:
            _dict['CountryCode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Address from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "StreetAddress": obj.get("StreetAddress") if obj.get("StreetAddress") is not None else None,
                        "StreetAddress2": obj.get("StreetAddress2") if obj.get("StreetAddress2") is not None else None,
                        "CityName": obj.get("CityName") if obj.get("CityName") is not None else None,
                        "ProvinceCode": obj.get("ProvinceCode") if obj.get("ProvinceCode") is not None else None,
                        "ProvinceName": obj.get("ProvinceName") if obj.get("ProvinceName") is not None else None,
                        "PostalCode": obj.get("PostalCode") if obj.get("PostalCode") is not None else None,
                        "CountryCode": obj.get("CountryCode") if obj.get("CountryCode") is not None else None
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class CallDetailReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    CAMPAIGNNAME = 'CampaignName'

    ADGROUPNAME = 'AdGroupName'

    STARTTIME = 'StartTime'

    ENDTIME = 'EndTime'

    DURATION = 'Duration'

    AREACODE = 'AreaCode'

    CITY = 'City'

    STATE = 'State'

    ACCOUNTID = 'AccountId'

    CAMPAIGNID = 'CampaignId'

    ADGROUPID = 'AdGroupId'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of CallDetailReportColumn from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ProductNegativeKeywordConflictReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    ADGROUPSTATUS = 'AdGroupStatus'

    MERCHANTPRODUCTID = 'MerchantProductId'

    TITLE = 'Title'

    ADGROUPCRITERIONID = 'AdGroupCriterionId'

    PRODUCTGROUP = 'ProductGroup'

    NEGATIVEKEYWORDID = 'NegativeKeywordId'

    NEGATIVEKEYWORD = 'NegativeKeyword'

    NEGATIVEKEYWORDLISTID = 'NegativeKeywordListId'

    CONFLICTLEVEL = 'ConflictLevel'

    NEGATIVEKEYWORDMATCHTYPE = 'NegativeKeywordMatchType'

    CAMPAIGNTYPE = 'CampaignType'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProductNegativeKeywordConflictReportColumn from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import bulk_str

class BulkAccountNegativeKeywordList(_SingleRecordBulkEntity):
    """ Represents an account negative keyword list association that can be read or written in a bulk file.

    This class exposes the :attr:`.BulkAccountNegativeKeywordList.account_negative_keyword_list` property that can be read and
    written as fields of the account negative keyword list association record in a bulk file.

    For more information, see account negative keyword list association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, account_negative_keyword_list=None):
        super(BulkAccountNegativeKeywordList, self).__init__()

        self._status = status
        self._account_negative_keyword_list = account_negative_keyword_list

    @property
    def account_negative_keyword_list(self):
        """ The account negative keyword list association.

        see account negative keyword list association at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._account_negative_keyword_list

    @account_negative_keyword_list.setter
    def account_negative_keyword_list(self, account_negative_keyword_list):
        self._account_negative_keyword_list = account_negative_keyword_list

    @property
    def status(self):
        """ The status of the account negative keyword list association.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.account_negative_keyword_list.Id),
            csv_to_field=lambda c, v: setattr(c.account_negative_keyword_list, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: c.account_negative_keyword_list.Name,
            csv_to_field=lambda c, v: setattr(c.account_negative_keyword_list, 'Name', v)
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._account_negative_keyword_list, 'account_negative_keyword_list')
        self.convert_to_values(row_values, BulkAccountNegativeKeywordList._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._account_negative_keyword_list = _CAMPAIGN_OBJECT_FACTORY_V13.create('AccountNegativeKeywordList')
        self._account_negative_keyword_list.Type = 'AccountNegativeKeywordList'
        row_values.convert_to_entity(self, BulkAccountNegativeKeywordList._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAccountNegativeKeywordList, self).read_additional_data(stream_reader)

class BulkAccountNegativeKeywordListAssociation(_SingleRecordBulkEntity):
    """ Represents an account negative keyword list association that is assigned to a campaign. Each account negative keyword list association can be read or written in a bulk file.

    This class exposes the :attr:`BulkAccountNegativeKeywordListAssociation.SharedEntityAssociation` property that can be read
    and written as fields of the account negative keyword list association record in a bulk file.

    For more information, see account negative keyword list association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, status=None, shared_entity_association=None):
        super(BulkAccountNegativeKeywordListAssociation, self).__init__()

        self._shared_entity_association = shared_entity_association
        self._status = status

    @property
    def status(self):
        """ The status of the account negative keyword list association.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def shared_entity_association(self):
        """ The campaign and account negative keyword list association identifiers.

        see Campaign account negative keyword list association at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._shared_entity_association

    @shared_entity_association.setter
    def shared_entity_association(self, shared_entity_association):
        self._shared_entity_association = shared_entity_association

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: c.status,
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.shared_entity_association.SharedEntityId),
            csv_to_field=lambda c, v: setattr(c.shared_entity_association, 'SharedEntityId', int(v))
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.shared_entity_association.EntityId),
            csv_to_field=lambda c, v: setattr(c.shared_entity_association, 'EntityId', int(v))
        ),
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._shared_entity_association, 'shared_entity_association')
        self.convert_to_values(row_values, BulkAccountNegativeKeywordListAssociation._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        self._shared_entity_association = _CAMPAIGN_OBJECT_FACTORY_V13.create('SharedEntityAssociation')
        self._shared_entity_association.EntityType = 'Account'
        self._shared_entity_association.SharedEntityType = 'NegativeKeywordList'
        row_values.convert_to_entity(self, BulkAccountNegativeKeywordListAssociation._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkAccountNegativeKeywordListAssociation, self).read_additional_data(stream_reader)


